diff -Npu mythtv-svn/libs/libmyth/libmyth.pro mythtv/libs/libmyth/libmyth.pro
--- mythtv-svn/libs/libmyth/libmyth.pro	2006-03-14 20:02:36.000000000 -0800
+++ mythtv/libs/libmyth/libmyth.pro	2006-04-08 12:55:41.000000000 -0700
@@ -17,7 +17,7 @@ HEADERS += mythdialogs.h audiooutput.h h
 HEADERS += uilistbtntype.h generictree.h screensaver.h
 HEADERS += managedlist.h DisplayRes.h volumebase.h audiooutputbase.h
 HEADERS += dbsettings.h screensaver-null.h output.h visual.h
-HEADERS += langsettings.h audiooutputnull.h
+HEADERS += langsettings.h audiooutputnull.h mythsocket.h
 HEADERS += DisplayResScreen.h util-x11.h mythdeque.h qmdcodec.h
 HEADERS += exitcodes.h virtualkeyboard.h mythobservable.h mythevent.h
 
@@ -31,7 +31,7 @@ SOURCES += volumecontrol.cpp volumebase.
 SOURCES += dbsettings.cpp screensaver.cpp screensaver-null.cpp output.cpp
 SOURCES += langsettings.cpp mythdbcon.cpp audiooutputnull.cpp
 SOURCES += DisplayResScreen.cpp util-x11.cpp qmdcodec.cpp
-SOURCES += virtualkeyboard.cpp mythobservable.cpp
+SOURCES += virtualkeyboard.cpp mythobservable.cpp mythsocket.cpp
 
 INCLUDEPATH += ../libmythsamplerate ../libmythsoundtouch ../.. ../
 DEPENDPATH += ../libmythsamplerate ../libmythsoundtouch ../ ../libmythui
diff -Npu mythtv-svn/libs/libmyth/mythcontext.h mythtv/libs/libmyth/mythcontext.h
--- mythtv-svn/libs/libmyth/mythcontext.h	2006-04-02 19:05:13.000000000 -0700
+++ mythtv/libs/libmyth/mythcontext.h	2006-04-08 12:55:41.000000000 -0700
@@ -99,6 +99,8 @@ class MythContextPrivate;
       "MHEG debugging messages")                 \
     F(VB_UPNP,      0x00080000, "upnp",      1,  \
       "upnp debugging messages")                 \
+    F(VB_SOCKET,    0x00100000, "socket",    1,  \
+      "socket debugging messages")               \
     F(VB_TIMESTAMP, 0x80000000, "timestamp", 1,  \
       "Conditional data driven messages")        \
     F(VB_NONE,      0x00000000, "none",      0,  \
diff -Npu mythtv-svn/libs/libmyth/mythsocket.cpp mythtv/libs/libmyth/mythsocket.cpp
--- mythtv-svn/libs/libmyth/mythsocket.cpp	1969-12-31 16:00:00.000000000 -0800
+++ mythtv/libs/libmyth/mythsocket.cpp	2006-04-08 21:47:47.000000000 -0700
@@ -0,0 +1,633 @@
+#include <qapplication.h>
+#include <qstring.h>
+#include "mythcontext.h"
+#include "util.h"
+#include "mythsocket.h"
+
+#define LOC QString("MythSocket(%1:%2): ").arg((unsigned long)this, 0, 16).arg(this->socket())
+
+const uint MythSocket::kSocketBufferSize = 128000;
+
+MythSocket::MythSocket(QObject *parent, const char *name, int socket)
+    : QObject(this, name), QSocketDevice(QSocketDevice::Stream),
+      m_parent(parent), m_sn_read(NULL), m_state(Idle), m_addr(), 
+      m_port(0), m_ref_count(0)
+{
+    VERBOSE(VB_SOCKET, LOC + "new socket");
+
+    if (socket >= 0)
+        setSocket(socket);
+}
+
+MythSocket::MythSocket(QObject *parent, int socket)
+    : QObject(this, NULL), QSocketDevice(QSocketDevice::Stream),
+      m_parent(parent), m_sn_read(NULL), m_state(Idle), m_addr(), 
+      m_port(0), m_ref_count(0)
+{
+    VERBOSE(VB_SOCKET, LOC + "new socket");
+
+    if (socket >= 0)
+        setSocket(socket);
+}
+
+MythSocket::MythSocket(QObject *parent)
+    : QObject(this, NULL), QSocketDevice(QSocketDevice::Stream),
+      m_parent(parent), m_sn_read(NULL), m_state(Idle), m_addr(), 
+      m_port(0), m_ref_count(0)
+{
+    VERBOSE(VB_SOCKET, LOC + "new socket");
+}
+
+MythSocket::~MythSocket()
+{
+    close();
+    m_ref_count--;
+   
+    if (m_ref_count >= 0)
+        VERBOSE(VB_IMPORTANT, LOC + "socket deleted while still referenced");
+
+    VERBOSE(VB_SOCKET, LOC + "delete socket");
+}
+
+void MythSocket::deleteLater(void)
+{
+    VERBOSE(VB_SOCKET, LOC + "deleteLater");
+    close();
+    QObject::deleteLater();
+}
+
+void MythSocket::UpRef(void)
+{
+    m_ref_count++;
+    VERBOSE(VB_SOCKET, LOC + QString("UpRef: %1").arg(m_ref_count));
+}
+
+bool MythSocket::DownRef(void)
+{
+    m_ref_count--;
+    VERBOSE(VB_SOCKET, LOC + QString("DownRef: %1").arg(m_ref_count));
+    if (m_ref_count < 0) 
+    {
+        delete this;
+        return true;
+    }
+    return false;
+}
+
+void MythSocket::setState(const State state)
+{
+    if (state != m_state)
+        VERBOSE(VB_SOCKET, LOC + QString("state change %1 -> %2")
+                .arg(stateToString(m_state)).arg(stateToString(state))); 
+
+    m_state = state;
+}
+
+QString MythSocket::stateToString(const State state)
+{
+    switch(state)
+    {
+        case Connected:
+            return QString("Connected");
+        case Connecting:
+            return QString("Connecting");
+        case Idle:
+            return QString("Idle");
+        default:
+            return QString("Invalid State: %1").arg(state);
+    }
+}
+
+QString MythSocket::errorToString(const Error error)
+{
+    switch(error)
+    {
+        case NoError:
+            return QString("NoError");
+        case AlreadyBound:
+            return QString("AlreadyBound");
+        case Inaccessible:
+            return QString("Inaccessible");
+        case NoResources:
+            return QString("NoResources");
+        case InternalError:
+            return QString("InternalError");
+        case Impossible:
+            return QString("Impossible");
+        case NoFiles:
+            return QString("NoFiles");
+        case ConnectionRefused:
+            return QString("ConnectionRefused");
+        case NetworkFailure:
+            return QString("NetworkFailure");
+        case UnknownError:
+            return QString("UnknownError");
+        default:
+            return QString("Invalid error: %1").arg(error);
+    }
+}
+
+void MythSocket::setSocket(int socket, Type type)
+{
+    if (socket < 0)
+    {
+        VERBOSE(VB_SOCKET, LOC + "setSocket called with invalid socket");
+        return;
+    }
+
+    if (state() == Connected)
+    {
+        VERBOSE(VB_SOCKET, LOC + 
+                "setSocket called while in Connected state, closing");
+        close();
+    }
+
+    QSocketDevice::setSocket(socket, type);
+    setBlocking(false);
+    setState(Connected);
+    setupReadNotify();
+}
+
+void MythSocket::setupReadNotify(void)
+{
+    qApp->lock();
+    m_sn_read  = new QSocketNotifier(socket(), 
+                                     QSocketNotifier::Read, m_parent);
+    QObject::connect(m_sn_read,  SIGNAL(activated(int)), this, 
+                     SLOT(notifyRead()));
+    qApp->unlock();
+}
+
+void MythSocket::close(void)
+{
+
+    qApp->lock();
+    if (m_sn_read)
+    {
+        delete(m_sn_read);
+        m_sn_read = NULL;
+    }
+    qApp->unlock();
+  
+    setState(Idle);
+    QSocketDevice::close();
+}
+
+Q_LONG MythSocket::readBlock(char *data, Q_ULONG len)
+{
+    VERBOSE(VB_SOCKET, LOC + "readblock called");
+
+    if(state() != Connected)
+    {
+        VERBOSE(VB_SOCKET, LOC + "readBlock called while not in " 
+                "connected state");
+        return -1;
+    }
+
+     Q_LONG rval = QSocketDevice::readBlock(data, len);
+     if (rval == 0)
+     {
+         close();
+         VERBOSE(VB_SOCKET, LOC + "emitting connectionClosed()");
+         emit connectionClosed();
+     }
+     return rval;
+}
+
+/** \fn writeBlock(char*, Q_ULONG)
+ *  \brief Attempt to write 'len' bytes to socket
+ *  \return actual bytes written
+ */
+Q_LONG MythSocket::writeBlock(const char *data, Q_ULONG len)
+{
+    if(state() != Connected)
+    {
+        VERBOSE(VB_SOCKET, LOC + "writeBlock called while not in " 
+                "connected state");
+        return -1;
+    }
+
+    Q_LONG rval = QSocketDevice::writeBlock(data, len);
+
+    // see if socket went away
+    if (!isValid() || error() != QSocketDevice::NoError)
+    {
+        close();
+        VERBOSE(VB_SOCKET, LOC + "emitting connectionClosed()");
+        emit connectionClosed();
+        return -1;
+    }
+    return rval;
+}
+
+bool MythSocket::writeStringList(QStringList &list)
+{
+    if (list.size() <= 0)
+    {
+        VERBOSE(VB_IMPORTANT, LOC + 
+                "writeStringList: Error, invalid string list.");
+        return false;
+    }
+
+    if (state() != Connected)
+    {
+        VERBOSE(VB_IMPORTANT, LOC +
+                "writeStringList: Error, called with unconnected socket.");
+        return false;
+    }
+
+    QString str = list.join("[]:[]");
+    if (str == QString::null)
+    {
+        VERBOSE(VB_IMPORTANT, LOC +
+                "writeStringList: Error, joined null string.");
+        return false;
+    }
+
+    QCString utf8 = str.utf8();
+    int size = utf8.length();
+    int written = 0;
+
+    QCString payload;
+    payload = payload.setNum(size);
+    payload += "        ";
+    payload.truncate(8);
+    payload += utf8;
+    size = payload.length();
+
+    if ((print_verbose_messages & VB_NETWORK) != 0)
+    {
+        QString msg = QString("write -> %1 %2")
+            .arg(socket(), 2).arg(payload);
+
+        if ((print_verbose_messages != VB_ALL) && msg.length() > 88)
+        {
+            msg.truncate(85);
+            msg += "...";
+        }
+        VERBOSE(VB_NETWORK, msg);
+    }
+
+    unsigned int errorcount = 0;
+    while (size > 0)
+    {
+        if (state() != Connected)
+        {
+            VERBOSE(VB_IMPORTANT, LOC +
+                    "writeStringList: Error, socket went unconnected.");
+            return false;
+        }
+
+        int temp = writeBlock(payload.data() + written, size);
+        if (temp > 0)
+        {
+            written += temp;
+            size -= temp;
+        }
+        else if (temp < 0 && error() != QSocketDevice::NoError)
+        {
+            VERBOSE(VB_IMPORTANT, LOC +
+                    QString("writeStringList: Error, writeBlock failed. (%1)")
+                    .arg(errorToString()));
+            return false;
+        } 
+        else if (temp <= 0)
+        {
+            errorcount++;
+            if (errorcount > 50)
+            {          
+                VERBOSE(VB_GENERAL, LOC +
+                        "writeStringList: No data written on writeBlock");
+                return false;
+            }
+            usleep(500);
+        }
+    }
+    return true;
+}
+
+/** \fn writeData(char*,Q_ULONG)
+ *  \brief Write len bytes to data to socket
+ *  \return true if entire len of data is written
+ */
+bool MythSocket::writeData(const char *data, Q_ULONG len)
+{
+    if (state() != Connected)
+    {
+        VERBOSE(VB_IMPORTANT, LOC +
+                "writeData: Error, called with unconnected socket.");
+        return false;
+    }
+
+    Q_ULONG written = 0;
+    uint zerocnt = 0;
+
+    while (written < len)
+    {
+        Q_LONG btw = len - written >= kSocketBufferSize ?
+                                       kSocketBufferSize : len - written;
+        Q_LONG sret = writeBlock(data + written, btw);
+        if (sret > 0)
+        {
+            zerocnt = 0;
+            written += sret;
+        }
+        else if (!isValid())
+        {
+            VERBOSE(VB_IMPORTANT, LOC +
+                    "writeData: Error, socket went unconnected");
+            close();
+            return false;
+        }
+        else if (sret < 0 && error() != QSocketDevice::NoError)
+        {
+            VERBOSE(VB_IMPORTANT, LOC +
+                    QString("writeData: Error, writeBlock: %1")
+                    .arg(errorToString()));
+            close();
+            return false;
+        }
+        else
+        {
+            zerocnt++;
+            if (zerocnt > 200)
+            {
+                VERBOSE(VB_IMPORTANT, LOC +
+                        "writeData: Error, zerocnt timeout");
+                return false;
+            }
+            usleep(1000);
+         }
+    }
+    return true;
+}
+
+bool MythSocket::readStringList(QStringList &list, bool quickTimeout)
+{
+    list.clear();
+
+    if (state() != Connected)
+    {
+        VERBOSE(VB_IMPORTANT, LOC +
+                "readStringList: Error, called with unconnected socket.");
+        return false;
+    }
+
+    MythTimer timer;
+    timer.start();
+    int elapsed = 0;
+
+    while (waitForMore(5) < 8)
+    {
+        elapsed = timer.elapsed();
+        if (!quickTimeout && elapsed >= 300000)
+        {
+            VERBOSE(VB_GENERAL, LOC + "readStringList: Error, timeout.");
+            close();
+            return false;
+        }
+        else if (quickTimeout && elapsed >= 20000)
+        {
+            VERBOSE(VB_GENERAL, LOC + 
+                    "readStringList: Error, timeout (quick).");
+            close();
+            return false;
+        }
+    }
+
+    QCString sizestr(8 + 1);
+    if (readBlock(sizestr.data(), 8) < 0)
+    {
+        VERBOSE(VB_GENERAL, LOC +
+                QString("readStringList: Error, readBlock return error (%1)")
+                .arg(errorToString()));
+        close();
+        return false;
+    }
+
+    sizestr = sizestr.stripWhiteSpace();
+    Q_LONG btr = sizestr.toInt();
+
+    if (btr < 1)
+    {
+        int pending = bytesAvailable();
+        QCString dump(pending + 1);
+        readBlock(dump.data(), pending);
+        VERBOSE(VB_IMPORTANT, LOC +
+                QString("Protocol error: '%1' is not a valid size " 
+                        "prefix. %2 bytes pending.")
+                        .arg(sizestr).arg(pending));
+        return false;
+    }
+
+    QCString utf8(btr + 1);
+
+    Q_LONG read = 0;
+    int errmsgtime = 0;
+    timer.start();
+    
+    while (btr > 0)
+    {
+        Q_LONG sret = readBlock(utf8.data() + read, btr);
+        if (sret > 0)
+        {
+            read += sret;
+            btr -= sret;
+            if (btr > 0)
+            {
+                timer.start();
+            }    
+        } 
+        else if (sret < 0 && error() != QSocketDevice::NoError)
+        {
+            VERBOSE(VB_GENERAL, LOC +
+                    QString("readStringList: Error, readBlock %1")
+                    .arg(errorToString()));
+            close();
+            return false;
+        }
+        else if (!isValid())
+        {
+            VERBOSE(VB_IMPORTANT, LOC +
+                    "readStringList: Error, socket went unconnected");
+            close();
+            return false;
+        }
+        else
+        {
+            elapsed = timer.elapsed();
+            if (elapsed  > 10000)
+            {
+                if ((elapsed - errmsgtime) > 10000)
+                {
+                    errmsgtime = elapsed;
+                    VERBOSE(VB_GENERAL, LOC +
+                            QString("readStringList: Waiting for data: %1 %2")
+                            .arg(read).arg(btr));
+                }                            
+            }
+            
+            if (elapsed > 100000)
+            {
+                VERBOSE(VB_GENERAL, LOC + 
+                        "Error, readStringList timeout (readBlock)");
+                return false;
+            }
+            
+            // usleep(500);
+        }
+    }
+
+    QString str = QString::fromUtf8(utf8.data());
+
+    QCString payload;
+    payload = payload.setNum(str.length());
+    payload += "        ";
+    payload.truncate(8);
+    payload += str;
+    
+    if ((print_verbose_messages & VB_NETWORK) != 0)
+    {
+        QString msg = QString("read  <- %1 %2").arg(socket(), 2)
+                                               .arg(payload);
+
+        if ((print_verbose_messages != VB_ALL) && msg.length() > 88)
+        {
+            msg.truncate(85);
+            msg += "...";
+        }
+        VERBOSE(VB_NETWORK, msg);
+    }
+
+    list = QStringList::split("[]:[]", str, true);
+
+    return true;
+}
+
+/** \fn connect(QString &, Q_UINT16)
+ *  \brief connect to host
+ *  \return true on success
+ */
+bool MythSocket::connect(const QString &host, Q_UINT16 port)
+{
+    QHostAddress hadr;
+    hadr.setAddress(host);
+    return MythSocket::connect(hadr, port);
+}
+
+/** \fn connect(QHostAddress &, Q_UINT16)
+ *  \brief connect to host
+ *  \return true on success
+ */
+bool MythSocket::connect(const QHostAddress &addr, Q_UINT16 port)
+{
+    if (state() == Connected)
+    {
+        VERBOSE(VB_SOCKET, LOC +
+                "connect() called with already open socket, closing");
+        close();
+    }
+
+    VERBOSE(VB_SOCKET, LOC + QString("attempting connect() to (%1:%2)")
+            .arg(addr.toString()).arg(port));
+
+    if (!QSocketDevice::connect(addr, port))
+    {
+        VERBOSE(VB_SOCKET, LOC + QString("connect() failed (%1)")
+                .arg(errorToString()));
+        setState(Idle); // ?
+        return false;
+    }
+        
+    setReceiveBufferSize(kSocketBufferSize);
+    if (state() == Connecting)
+    {
+        setState(Connected);
+        VERBOSE(VB_SOCKET, LOC + "emitting connected()");
+        emit connected();
+    }
+    else
+    {
+        setState(Connected);
+    }        
+    setupReadNotify();
+    return true;
+}
+
+/** \fn connectToHost(QString &, Q_UINT16)
+ *  \brief connect to host in background
+ */
+void MythSocket::connectToHost(const QString &host, Q_UINT16 port)
+{
+    QHostAddress hadr;
+    hadr.setAddress(host);
+    MythSocket::connectToHost(hadr, port);
+}
+
+/** \fn connectToHost(QHostAddress &, Q_UINT16)
+ *  \brief connect to host in background
+ */
+void MythSocket::connectToHost(const QHostAddress &addr, Q_UINT16 port)
+{
+    m_addr = addr;
+    m_port = port;
+
+    if (state() != Idle)
+    {
+        VERBOSE(VB_SOCKET, LOC +
+                "connectToHost() called with already open socket, closing");
+        close();
+    }
+
+    VERBOSE(VB_SOCKET, LOC + 
+            QString("starting background connection to (%1:%2)")
+            .arg(addr.toString()).arg(port));
+
+    setState(Connecting);
+
+    m_connect_timer = new QTimer(this);
+    QObject::connect(m_connect_timer, SIGNAL(timeout()), SLOT(tryConnect()));
+    m_connect_timer->start(0, TRUE);
+}
+
+void MythSocket::notifyRead(void)
+{
+    if (m_sn_read)
+        m_sn_read->setEnabled(false);
+
+    Q_LONG avail = bytesAvailable();
+    VERBOSE(VB_SOCKET, LOC + QString("notifyRead: bytes avail: %1").arg(avail));
+
+    //  see if connection closed
+    if (!avail)
+    {
+        VERBOSE(VB_SOCKET, LOC + "notifyRead: connection closed");
+        close();
+        VERBOSE(VB_SOCKET, LOC + "emitting connectionClosed()");
+        emit connectionClosed();
+        return;
+    }
+
+    VERBOSE(VB_SOCKET, LOC + "emitting readyRead()");
+    emit readyRead();
+
+    if (m_sn_read)
+        m_sn_read->setEnabled(true);
+}
+
+void MythSocket::tryConnect(void)
+{
+    delete(m_connect_timer);
+    m_connect_timer = NULL;
+
+    if (state() == Connecting)
+    {
+        if (!MythSocket::connect(m_addr, m_port))
+        {
+            // TODO: add retry?
+            emit connectionFailed();
+            VERBOSE(VB_SOCKET, LOC + "emitting connectionFailed()");
+            return;
+        }
+    }
+}
diff -Npu mythtv-svn/libs/libmyth/mythsocket.h mythtv/libs/libmyth/mythsocket.h
--- mythtv-svn/libs/libmyth/mythsocket.h	1969-12-31 16:00:00.000000000 -0800
+++ mythtv/libs/libmyth/mythsocket.h	2006-04-08 20:12:56.000000000 -0700
@@ -0,0 +1,77 @@
+#ifndef MYTHSOCKET_H
+#define MYTHSOCKET_H
+
+#include <qsocketdevice.h>
+#include <qsocketnotifier.h>
+#include <qtimer.h>
+class QHostAddress;
+
+class MythSocket : public QObject, public QSocketDevice
+{
+  Q_OBJECT
+
+  public:
+    MythSocket(QObject *parent, const char *name, int socket);
+    MythSocket(QObject *parent, int socket);
+    MythSocket(QObject *parent = NULL);
+   ~MythSocket();
+
+    enum State {
+        Connected,
+        Connecting,
+        Idle
+    };
+
+    void close(void);
+    void deleteLater(void);
+ 
+    void UpRef(void);
+    bool DownRef(void);
+
+    State   state(void) { return m_state; }
+    QString stateToString(void) { return stateToString(state()); }
+    QString stateToString(const State state);
+
+    QString errorToString(void) { return errorToString(error()); }
+    QString errorToString(const Error error);
+
+    void setSocket(int socket, Type type = QSocketDevice::Stream);
+    Q_LONG  readBlock(char *data, Q_ULONG len);
+    Q_LONG writeBlock(const char *data, Q_ULONG len);
+
+    bool  readStringList(QStringList &list, bool quickTimeout = false);
+    bool writeStringList(QStringList &list);
+    bool writeData(const char *data, Q_ULONG len);
+
+    bool connect(const QHostAddress &addr, Q_UINT16 port);
+    bool connect(const QString &host, Q_UINT16 port);
+    void connectToHost(const QHostAddress &addr, Q_UINT16 port);
+    void connectToHost(const QString &host, Q_UINT16 port);
+
+  private:
+    void setState(const State state);
+    void setupReadNotify(void);
+
+    QObject          *m_parent;
+    QSocketNotifier  *m_sn_read;
+    QTimer           *m_connect_timer;
+
+    State             m_state;
+    QHostAddress      m_addr;
+    Q_UINT16          m_port;
+    int               m_ref_count;
+
+    static const uint kSocketBufferSize;
+
+  signals:
+    void connected();
+    void connectionClosed();
+    void connectionFailed();
+    void readyRead();
+
+  private slots:
+    void notifyRead();
+    void tryConnect();
+};
+
+#endif
